'    Copyright 2007, Leonardo Cecchi
'
'    This file is part of Project Manager.
'
'    Project Manager is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    Project Manager is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with Project Manager; if not, write to the Free Software
'    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "BE_LineFile.bi"

'----------------------------------
' Clear this line file
'----------------------------------
sub LF_Clear(Self as LineFile ptr)
	self->lines=0
	self->sorted=-1
end sub

'----------------------------------
' Read from file
'----------------------------------
sub LF_Read(Self as LineFile ptr, fileName as String)
	dim as Integer fileHandle, i
	
	LF_Clear(Self)
	fileHandle=Freefile()
	open fileName For input as #fileHandle

	self->lines=1
	do until eof(fileHandle)
		line input #fileHandle, self->contents(self->lines)
		self->lines=self->lines+1
	loop
	self->lines=self->lines-1

	self->sorted=0

	close #fileHandle
end sub

'----------------------------------
' Init this line file
'----------------------------------
sub LF_Init(Self as LineFile ptr)
	LF_Clear(Self)
end sub

'----------------------------------
' Sort this config file.
' Useful for binary search in 
' configuration and translation
' files
'----------------------------------
sub LF_Sort(Self as LineFile ptr)
	dim i as Integer
	
	Self->sorted=0
	do until Self->sorted
		Self->sorted=-1
		for i=2 to self->lines
			if self->contents(i)<self->contents(i-1) then
				dim t as String
				t=self->contents(i-1)
				self->contents(i-1)=self->contents(i)
				self->contents(i)=t
				Self->sorted=0
			end if
		next
	loop

end sub

'----------------------------------
' Find the position of a key
' in this file. If the key doesn't
' exist return -1
'----------------------------------
function LF_Find(Self as LineFile ptr, byval keyOrig as String) as Integer	
	dim as Integer upBound, lowBound, midBound, i
	dim as String compareKey, keyPlusEqual
	
	if not self->sorted or self->lines=0 then
		return -1
	end if
	
	lowBound=1
	midBound=1
	upBound=self->lines
	
	keyPlusEqual=keyOrig+"="
	do while lowBound<=upBound
		midBound=(upBound+lowBound)/2
		compareKey = left(self->contents(midBound), len(keyPlusEqual))
		if compareKey=keyPlusEqual then
			lowBound=midBound
			upBound=midBound
			exit do
		elseif compareKey<keyPlusEqual then
			lowBound=midBound+1
		elseif keyPlusEqual<compareKey then
			upBound=midBound-1
		end if
	loop
	
	midBound=(upBound+lowBound)/2
	if midBound>=1 and midBound<=self->lines then
		compareKey = left(self->contents(midBound), len(keyPlusEqual))
		if compareKey=keyPlusEqual then
			return midBound
		else
			return -1
		end if
	else
		return -1
	end if	
end function

'----------------------------------
' Find a key in this file.
' The file must be sorted. If the
' file isn't sorted this returns 
' "<not sorted>"
' If the key doesn't exists
' return default
'----------------------------------
function LF_Get(Self as LineFile ptr, key as String, default as String = "") as String
	dim as Integer midBound
	dim as String compareKey

	if not self->sorted then
		return "<not sorted>"
	end if
	
	midBound=LF_Find(Self, key)
	
	if midBound<>-1 then
		return mid(self->contents(midBound), len(key)+2)
	else
		return default
	end if
end function

'----------------------------------
' Set a key in this file.
' The file must be sorted. If the
' file isn't sorted this is a no-op
'----------------------------------
sub LF_Set(Self as LineFile ptr, key as String, value as String) 
	dim as Integer midBound
	dim as String compareKey
	
	if not self->sorted then
		return 
	end if
	
	midBound=LF_Find(Self, key)
	if midBound<>-1 then
		self->contents(midBound)=key+"="+value
	elseif self->lines <= MAX_LINES then
		self->lines=self->lines+1
		self->contents(self->lines)=key+"="+value
	end if
	
	LF_Sort(self)
end sub

'----------------------------------
' Write to file
'----------------------------------
sub LF_Write(Self as LineFile ptr, fileName as String)
	dim as Integer fileHandle, i
	
	fileHandle=Freefile()
	open fileName For output as #fileHandle

	for i=1 to self->lines
		print #fileHandle, self->contents(i)
	next
	
	close #fileHandle
end sub
