'    Copyright 2007, Leonardo Cecchi
'
'    This file is part of Project Manager.
'
'    Project Manager is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    Project Manager is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with Project Manager; if not, write to the Free Software
'    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include once "BE_Project.bi"
#include once "BE_Utils.bi"
#include once "Translations.bi"

' ----------------------------------------------------
' Nome del file di configurazione
' ----------------------------------------------------
function P_Configuration_File_Name(self as Project ptr) as String
	return (self->projectDirectory) + (self->separator) + CONF_FILE_NAME
end function

' ----------------------------------------------------
' Legge la configurazione del progetto dalla directory
' ----------------------------------------------------
sub P_Read_Conf(self as Project ptr)
	dim as String confFile = P_Configuration_File_Name(self)
	lf_clear(@self->configuration)
	if FileExists(confFile)<>0 then
		print "Reading configuration file",confFile
		lf_read(@self->configuration, confFile)
		lf_sort(@self->configuration)
	else
		' Configuration default values
		LF_Set(@self->configuration,EDITOR_KEY,DEF_EDITOR)
		LF_Set(@self->configuration,EDITOR_GOTO_KEY,DEF_EDITOR_GOTO)
		LF_Set(@self->configuration,COMPILER_KEY, DEF_COMPILER)
		LF_Set(@self->configuration,DEBUGGER_KEY, DEF_DEBUGGER)
		LF_Set(@self->configuration,PDFLATEX_KEY, DEF_PDFLATEX)
		LF_Set(@self->configuration,PDF_KEY,DEF_PDF)
		LF_Set(@self->configuration,BASIC_MAIN_KEY,DEF_BASIC_MAIN)
		LF_Set(@self->configuration,LATEX_MAIN_KEY, DEF_LATEX_MAIN) 	
		LF_Set(@self->configuration,EXEC_COMMAND_KEY, DEF_EXEC_COMMAND) 	
		LF_Set(@self->configuration,TEMPORARY_FILE_KEY, DEF_TEMPORARY_FILE) 	
	end if
end sub

' -----------------------------------------------------
' Scrive la configurazione del progetto dalla directory
' -----------------------------------------------------
sub P_Write_Conf(self as Project ptr)
	dim as String confFile = P_Configuration_File_Name(self)
	lf_write(@self->configuration, confFile)
end sub

' ----------------------------------------------------
' Verifica un filtro. Questa funzione  stata scritta
' per evitare il comportamento strano della funzione
' "dir"
' ----------------------------------------------------
function P_Verify_Filter(fileName as  String, filter as String) as Integer
  dim i as integer
  dim dfileName as String, dFilter as String 
  dFileName=fileName
  dFilter=filter
  i=1
  while len(dFileName)>0
    if len(dFilter)=0 then
      return 0
    elseif left(dFilter,1)="?" then
      dFileName=mid(dFileName,2)
      dFilter=mid(dFilter,2)
    elseif left(dFilter,1)="*" then
      if P_Verify_Filter(mid(dFileName,2), dFilter) then
        return 1
      else
        dFilter=mid(dFilter,2)
      end if
    elseif left(dFileName,1)=left(dFilter,1) then 
      dFileName=mid(dFileName,2)
      dFilter=mid(dFilter,2)
    else
      return 0
    end if  
    i=i+1
  wend
  return len(dFilter)=0
end function

' ----------------------------------------------------
' Aggiunge una lista di files al progetto
' ----------------------------------------------------
sub P_Append_Files(self as Project ptr, _
                   list() as String, _
		   byref count as Integer, pattern as String)
	dim filter as String = self->projectDirectory + self->separator + pattern
	dim as integer ordered, i, first
	count=1
	first=1

	' Aggiunta dei files
	while count<=MAX_FILES
		if first then
			list(count)=dir(filter)
			first=0
		else
			list(count)=dir()
		end if
		
		if len(list(count))=0 then
			exit while
		end if
		
		if P_Verify_Filter(list(count), pattern) then
		  count=count+1
		else
		  list(count)=""
		end if
	wend
	count=count-1
	
	' Ordinamento dei files in base al loro prefisso
	' e non in base all'ordine del file
	dim as String pref1, pref2, name1, name2
	ordered=0
	while not ordered
		ordered=-1
		for i=2 to count
			SplitFileName(list(i-1), pref1, name1)
			SplitFileName(list(i), pref2, name2)
			if pref1>pref2 then
				dim t as String = list(i-1)
				list(i-1)=list(i)
				list(i)=t
				ordered=0
			end if	
		next
	wend
end sub

' ----------------------------------------------------
' Rilegge il progetto dalla directory
' ----------------------------------------------------
sub P_Refresh(self as Project ptr)
	chdir(self->ProjectDirectory)
	P_Append_Files(Self, Self->sourceFiles(), Self->sourceFilesCount, "*.bas")
	P_Append_Files(Self, Self->includeFiles(), Self->includeFilesCount, "*.bi")
	P_Append_Files(Self, Self->docFiles(), Self->docFilesCount, "*.txt")
	P_Append_Files(Self, Self->docFilesTex(), Self->docFilesTexCount, "*.tex")
	P_Read_Conf(self)
end sub

' ----------------------------------------------------
' Pulisce tutti i dati dal progetto
' ----------------------------------------------------
sub P_Clear(self as Project ptr)
	dim i as Integer
	
	self->projectDirectory=""
#ifdef __FB_LINUX__
	self->separator="/" 
#else	
	self->separator="\\"  
#endif
	self->includeFilesCount=0
	self->sourceFilesCount=0
	self->docFilesCount=0
	
	for i=1 to MAX_FILES
		self->sourceFiles(i)=""
		self->includeFiles(i)=""
		self->docFiles(i)=""
		self->docFilesTex(i)=""
	next
	
	LF_Clear(@self->configuration)
end sub

' ----------------------------------------------------
' Inizializza il progetto
' ----------------------------------------------------
sub P_Init(self as Project ptr)
	LF_Init(@self->configuration)	
	P_Clear(self)	
end sub

' ----------------------------------------------------
' Inizializza il progetto a partire da una
' directory
' ----------------------------------------------------
sub P_Init_From_Dir(self as Project ptr, directory as String)
	P_Init(self)
	self->projectDirectory=directory
	P_Refresh(self)
end sub

' ----------------------------------------------------
' Ritorna 1 se il progetto  vuoto e 0 altrimenti
' ----------------------------------------------------
function P_Empty(self as Project ptr) as integer
	if len(self->projectDirectory)=0 then
		return -1
	else
		return 0
	end if
end function

' ----------------------------------------------------
' Chiude il progetto riportandolo al progetto vuoto
' ----------------------------------------------------
sub P_Close(self as Project ptr) 
	P_Clear(self)
end sub

' ----------------------------------------------------
' Execute a command filling the output rows with
' the output of the command
' ----------------------------------------------------
sub P_CommandExecute(self as Project ptr, byval cmd as String)
	dim as Integer fileNo, openerror
	dim as String cmdRedirect,s 
	dim as String tempFile
	
	tempFile=LF_Get(@self->configuration, TEMPORARY_FILE_KEY, DEF_TEMPORARY_FILE)
	
	fileNo=FreeFile()

	cmdRedirect=cmd+" > "+ tempFile + " 2>&1 "
#ifndef __FB_LINUX__
	cmdRedirect="cmd.exe /c "+cmdRedirect
#endif
  print cmdRedirect
	
	if FileExists(tempFile) then
    kill(tempFile)
	end if
	shell(cmdRedirect)
	
	openerror=open (tempFile for input as #fileNo)
	
	if openerror then
	  InformationDialog(T_("Please check the configuration. Probably there is a wrong path"))
	else	
  	do while not eof(fileNo)
  		self->outputRowsCount=self->outputRowsCount+1
  		if self->outputRowsCount>=MAX_OUTPUT_ROWS then
  			self->outputRowsCount=MAX_OUTPUT_ROWS
  			exit do
  		end if
  		line input #fileNo, self->outputRows(self->outputRowsCount)
  		if len(self->outputRows(self->outputRowsCount))=0 then
  			self->outputRowsCount=self->outputRowsCount-1
  		end if
  	loop
  end if

	close #fileNo
end sub

' ----------------------------------------------------
' Compila il progetto riempiendo l'output
' ----------------------------------------------------
sub P_Compile(self as Project ptr)
	dim as Integer i
	dim as String cmdCompiler, mainFile
	
	chdir(self->projectDirectory)
	self->outputRowsCount=0
	mainFile=LF_GET(@self->configuration, BASIC_MAIN_KEY)
	cmdCompiler=LF_Get(@self->configuration, COMPILER_KEY)+" "

	' Prima di tutti il file main
	cmdCompiler=cmdCompiler+" -g "+mainFile+ " "

	' Compilo ogni file sorgente 
	for i=1 to self->sourceFilesCount
		dim as String currentFile=self->sourceFiles(i)
		if right(mainFile, len(currentFile))<>currentFile then
			cmdCompiler=cmdCompiler+" "+currentFile
		end if
	next
	
	P_CommandExecute(self, cmdCompiler)
end sub

' ----------------------------------------------------
' Esegue il progetto riempiendo l'output
' ----------------------------------------------------
sub P_Execute(self as Project ptr)
	dim as Integer i
	dim as String mainFile, cmd
	
	P_Compile(self)
	if self->outputRowsCount=0 then
	
		chdir(self->projectDirectory)
		self->outputRowsCount=0
		mainFile=LF_GET(@self->configuration, BASIC_MAIN_KEY)
		 
		if ucase(right(mainFile,4))=".BAS" then
			mainFile=left(mainFile,len(mainFile)-4)
		end if	
			
		' TODO Verificare che tutto funga su Windows
#ifndef __FB_LINUX__
		mainFile=mainFile+".exe"
#else
		mainFile="./"+mainFile
#endif
		
		cmd=LF_Get(@self->configuration, EXEC_COMMAND_KEY, DEF_EXEC_COMMAND)
		cmd=str_replace("<cmd>", mainFile, cmd)
		
		P_CommandExecute(self, cmd)
	end if
end sub

' ----------------------------------------------------
' Debug del progetto riempiendo l'output (NON COMPILA)
' ----------------------------------------------------
sub P_Debug(self as Project ptr)
	dim as Integer i
	dim as String cmdDebugger, mainFile
	
	chdir(self->projectDirectory)
	self->outputRowsCount=0
	cmdDebugger=LF_Get(@self->configuration, DEBUGGER_KEY)
	mainFile=LF_GET(@self->configuration, BASIC_MAIN_KEY)
	 
	if ucase(right(mainFile,4))=".BAS" then
		mainFile=left(mainFile,len(mainFile)-4)
	end if	
		
	' TODO Verificare che tutto funga su Windows
#ifndef __FB_LINUX__
	mainFile=mainFile+".exe"
#endif
	
	P_CommandExecute(self, cmdDebugger+" "+mainFile)
end sub

' ----------------------------------------------------
' Ricerca delle TOD_O in un file
' ----------------------------------------------------
sub P_Grep_File(self as Project ptr, fileName as String, mark as String)
	dim as Integer fileNo, rowNumber
	dim as String s
	
	fileNo=FreeFile()
	open fileName for input as #fileNo
	
	rowNumber=1
	while not eof(fileNo)
		line input #fileNo, s
		if self->outputRowsCount<MAX_OUTPUT_ROWS and instr(s, mark)<>0 then
			dim as String k = fileName+"("+str(rowNumber)+")"+s
			self->outputRows(self->outputRowsCount)=k
			self->outputRowsCount=self->outputRowsCount+1
		end if
		rowNumber=rowNumber+1
	wend
	
	close #fileNo
end sub

' ----------------------------------------------------
' Ricerca di una stringa in tutti i files
' ----------------------------------------------------
sub P_Grep_All_Files(self as Project ptr, mark as String)
	dim as Integer i
	
	chdir(self->projectDirectory)
	self->outputRowsCount=1
	
	for i=1 to self->sourceFilesCount
		P_Grep_File(self, self->sourceFiles(i), mark)
	next
	
	for i=1 to self->includeFilesCount
		P_Grep_File(self, self->includeFiles(i), mark)
	next
	
	for i=1 to self->docFilesCount
		P_Grep_File(self, self->docFiles(i), mark)
	next

	for i=1 to self->docFilesTexCount
		P_Grep_File(self, self->docFilesTex(i), mark)
	next	
	self->outputRowsCount-=1
end sub

' ----------------------------------------------------
' Ricerca delle TOD_O
' ----------------------------------------------------
sub P_Todo_Grepper(self as Project ptr)
	P_Grep_All_Files(self, TOD_O_MARK)
end sub
