'    Copyright 2007, Leonardo Cecchi
'
'    This file is part of Project Manager.
'
'    Project Manager is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    Project Manager is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with Project Manager; if not, write to the Free Software
'    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include once "PM_Configuration.bi"

'---------------------------------
' Show to the user a file selection
' dialog to choose a file to put
' in the entry widget
'---------------------------------
sub ChooseFileCallback cdecl (byval widget as GtkWidget ptr, byval user_data as GtkWidget ptr)
	dim as GtkWidget ptr dialog
	
	dialog=gtk_file_chooser_dialog_new(T_("Select a file"), _
		0, GTK_FILE_CHOOSER_ACTION_OPEN, _
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 0)
	if gtk_dialog_run(GTK_DIALOG(dialog)) =	 GTK_RESPONSE_ACCEPT then
		dim p as zstring ptr
		p=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog))
#ifdef __FB_LINUX__
		gtk_entry_set_text(GTK_ENTRY(user_data), ""+(*p))
#else
		gtk_entry_set_text(GTK_ENTRY(user_data), """"+(*p)+"""")
#endif
		g_free(p)
	end if
	gtk_widget_destroy(dialog)
end sub

'---------------------------------
' Utility function to pack a label
' and an entry widget in a hbox
'---------------------------------
function UtilPackButtonFile(label as String, entry as GtkWidget ptr) as GtkWidget ptr
	dim as GtkWidget ptr box, labelW, button
	
	box=gtk_hbox_new(0,0)
	labelW=gtk_label_new(label)
	button=gtk_button_new_with_label(T_("Choose File..."))
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(@ChooseFileCallback), entry)

	gtk_box_pack_start(GTK_BOX(box), labelW, 0, 0, 1)
	gtk_box_pack_start(GTK_BOX(box), entry, 1, 1, 1)
	gtk_box_pack_start(GTK_BOX(box), button, 0, 0, 1)

	return box
end function

'---------------------------------
' Initialize a new configuration
' dialog
'---------------------------------
sub PM_Configuration_Init(Self as PM_Configuration ptr, parent as GtkWidget ptr)
	dim as GtkWidget ptr box, notebook, lbtab
	
	self->dialog=gtk_dialog_new_with_buttons(T_("Project Configuration"), _
		GTK_WINDOW(parent), _
		GTK_DIALOG_MODAL or GTK_DIALOG_DESTROY_WITH_PARENT, _
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, _
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 0)

	' Environment configuration

	box=gtk_vbox_new(0, 5)
		
	self->enEditor=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Editor: "), self->enEditor), 0, 0, 1)
		
	self->enEditorGoto=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Editor Goto: "), self->enEditorGoto), 0, 0, 1)
		
	self->enCompiler=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Compiler: "), self->enCompiler), 0, 0, 1)
		
	self->enDebugger=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Debugger: "), self->enDebugger), 0, 0, 1)
		
	self->enPdfLatex=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Pdflatex: "), self->enPdfLatex), 0, 0, 1)
		
	self->enPdf=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Pdf Reader: "), self->enPdf), 0, 0, 1)
		
	self->enTemporaryFile=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Temporary File: "), self->enTemporaryFile), 0, 0, 1)
		
	self->enExecCmd=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Execution Command: "), self->enExecCmd), 0, 0, 1)

	gtk_container_set_border_width(GTK_CONTAINER(box), 10)
	
	notebook=gtk_notebook_new()
	lbtab=gtk_label_new(T_("Environment Settings"))
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, lbtab)

	' Project configuration
	
	box=gtk_vbox_new(0, 5)
	
	self->enBasicMain=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("Project Main File: "), self->enBasicMain), 0, 0, 1)
	
	self->enLatexMain=gtk_entry_new()
	gtk_box_pack_start(GTK_BOX(box), UtilPackButtonFile(T_("LaTex Main File: "), self->enLatexMain), 0, 0, 1)
	
	gtk_container_set_border_width(GTK_CONTAINER(box), 10)
	lbtab=gtk_label_new(T_("Project Settings"))
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, lbtab)
	
	gtk_widget_show_all(notebook)
	
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(self->dialog)->vbox), notebook, 0, 0, 1)
end sub

'---------------------------------
' Read a configuration file
'---------------------------------
sub PM_Configuration_Read(Self as PM_Configuration ptr, configuration as LineFile ptr)
	gtk_entry_set_text(GTK_ENTRY(self->enEditor), LF_Get(configuration, EDITOR_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enEditorGoto), LF_Get(configuration, EDITOR_GOTO_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enCompiler), LF_Get(configuration, COMPILER_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enDebugger), LF_Get(configuration, DEBUGGER_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enPdflatex), LF_Get(configuration, PDFLATEX_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enPdf), LF_Get(configuration, PDF_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enBasicMain), LF_Get(configuration, BASIC_MAIN_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enLatexMain), LF_Get(configuration, LATEX_MAIN_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enTemporaryFile), LF_Get(configuration, TEMPORARY_FILE_KEY, " "))
	gtk_entry_set_text(GTK_ENTRY(self->enExecCmd), LF_Get(configuration, EXEC_COMMAND_KEY, " "))
end sub

'---------------------------------
' Update a configuration file
'---------------------------------
sub PM_Configuration_Update(Self as PM_Configuration ptr, configuration as LineFile ptr)
  dim basicMainFile as String
  dim separatore as String
  dim idx as Integer
  
	LF_Set(configuration, EDITOR_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enEditor))))
	LF_Set(configuration, EDITOR_GOTO_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enEditorGoto))))
	LF_Set(configuration, COMPILER_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enCompiler))))
	LF_Set(configuration, DEBUGGER_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enDebugger))))
	LF_Set(configuration, PDFLATEX_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enPdflatex))))
	LF_Set(configuration, PDF_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enPdf))))
	LF_Set(configuration, LATEX_MAIN_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enLatexMain))))
	LF_Set(configuration, EXEC_COMMAND_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enExecCmd))))
	LF_Set(configuration, TEMPORARY_FILE_KEY, ""+(*gtk_entry_get_text(GTK_ENTRY(self->enTemporaryFile))))
	
	' Il nome del file principale basic deve essere senza alcuna path
	basicMainFile=""+(*gtk_entry_get_text(GTK_ENTRY(self->enBasicMain)))
	if left(basicMainFile,1)="""" and right(basicMainFile,1)="""" and len(basicMainFile)>=2 then
	  basicMainFile=mid(basicMainFile,2)
	  basicMainFile=left(basicMainFile, len(basicMainFile)-1)
	end if
#ifdef __FB_LINUX__
  separatore="/"
#else
  separatore="\"
#endif

  idx=len(basicMainFile)-1
  while idx>0
    if mid(basicMainFile, idx, 1)=separatore then
      basicMainFile=mid(basicMainFile, idx+1)
      exit while
    end if
    idx=idx-1
  wend
      	
	LF_Set(configuration, BASIC_MAIN_KEY, basicMainFile)
end sub

'---------------------------------
' Run this configuration dialog
'---------------------------------
function PM_Configuration_Run(Self as PM_Configuration ptr) as Integer
	return gtk_dialog_run(GTK_DIALOG(self->dialog))
end function

'---------------------------------
' Destroy this configuration dialog
'---------------------------------
sub PM_Configuration_Destroy(Self as PM_Configuration ptr)
	gtk_widget_destroy(self->dialog)
end sub
