'    Copyright 2007, Leonardo Cecchi
'
'    This file is part of Project Manager.
'
'    Project Manager is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    Project Manager is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with Project Manager; if not, write to the Free Software
'    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include once "PM_Frame.bi"
#include once "ProjectManager.bi"

#define NULL 0

' ----------------------------------------------------
' Inizializzazione della toolbar
' ----------------------------------------------------
sub PMF_Toolbar_init(Self as PM_Frame ptr)
	dim as GtkWidget ptr toolbar
	dim as GtkToolItem ptr toolOpen, toolClose, toolCompile, toolRun, toolDebug, toolQuit, _
		sep1, sep2, toolRefresh, sep3, sep4, toolTodo
	dim as GtkTooltips ptr toolTips
	
	toolTips=gtk_tooltips_new()
	
	' Creazione toolbar e bottoni
	toolbar=gtk_toolbar_new()
	
	toolOpen=gtk_tool_button_new_from_stock(GTK_STOCK_OPEN)
	gtk_tool_item_set_tooltip(toolOpen, toolTips, T_("Open Project"), "")
	g_signal_connect(G_OBJECT(toolOpen), "clicked", G_CALLBACK(@PMF_Open_Project), Self)
	g_object_set(G_OBJECT(toolOpen), "label", T_("Open"), 0)
	
	toolClose=gtk_tool_button_new_from_stock(GTK_STOCK_CLOSE)
	gtk_tool_item_set_tooltip(toolClose, toolTips, T_("Close Project"), "")
	g_signal_connect(G_OBJECT(toolClose), "clicked", G_CALLBACK(@PMF_Close_Project), Self)
	g_object_set(G_OBJECT(toolClose), "label", T_("Close"), 0)
	
	sep1=gtk_separator_tool_item_new()

	toolRefresh=gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH)
	gtk_tool_item_set_tooltip(toolRefresh, toolTips, T_("Refresh Project"), "")
	g_signal_connect(G_OBJECT(toolRefresh), "clicked", G_CALLBACK(@PMF_Refresh_Project), Self)
	g_object_set(G_OBJECT(toolRefresh), "label", T_("Refresh"), 0)

	sep2=gtk_separator_tool_item_new()
	
	toolCompile=gtk_tool_button_new_from_stock(GTK_STOCK_JUSTIFY_FILL)
	gtk_tool_item_set_tooltip(toolCompile, toolTips, T_("Compile Project"), "")
	g_signal_connect(G_OBJECT(toolCompile), "clicked", G_CALLBACK(@PMF_Build), Self)
	g_object_set(G_OBJECT(toolCompile), "label", T_("Compile"), 0)
	
	toolRun=gtk_tool_button_new_from_stock(GTK_STOCK_EXECUTE)
	gtk_tool_item_set_tooltip(toolRun, toolTips, T_("Compile & Run Project"), "")
	g_signal_connect(G_OBJECT(toolRun), "clicked", G_CALLBACK(@PMF_Run), Self)
	g_object_set(G_OBJECT(toolRun), "label", T_("Run"), 0)

	toolDebug=gtk_tool_button_new_from_stock(GTK_STOCK_MEDIA_RECORD)
	gtk_tool_item_set_tooltip(toolDebug, toolTips, T_("Debug Project"), "")
	g_signal_connect(G_OBJECT(toolDebug), "clicked", G_CALLBACK(@PMF_Debug), Self)
	g_object_set(G_OBJECT(toolDebug), "label", T_("Debug"), 0)
	
	sep3=gtk_separator_tool_item_new()

	toolTodo=gtk_tool_button_new_from_stock(GTK_STOCK_APPLY)
	gtk_tool_item_set_tooltip(toolTodo, toolTips, T_("Fill TODO List"), "")
	g_signal_connect(G_OBJECT(toolTodo), "clicked", G_CALLBACK(@PMF_Todo), Self)	
	g_object_set(G_OBJECT(toolTodo), "label", T_("Todo"), 0)
	
	sep4=gtk_separator_tool_item_new()
	
	toolQuit=gtk_tool_button_new_from_stock(GTK_STOCK_QUIT)
	gtk_tool_item_set_tooltip(toolQuit, toolTips, T_("Quit"), "")
	g_signal_connect(G_OBJECT(toolQuit), "clicked", G_CALLBACK(@PMF_Close), Self)
	g_object_set(G_OBJECT(toolQuit), "label", T_("Quit"), 0)
	
	' Aggiunta dei bottoni alla toolbar
	gtk_toolbar_set_tooltips(GTK_TOOLBAR(toolbar), -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolOpen, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolClose, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), sep1, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolRefresh, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), sep2, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolCompile, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolRun, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolDebug, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), sep3, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolTodo, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), sep4, -1)
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), toolQuit, -1)
	
	gtk_box_pack_start(GTK_BOX(self->vbox), toolbar, 0, 0, 0)
end sub

' ----------------------------------------------------
' Creazione della barra dei menu
' ----------------------------------------------------
sub PMF_MenuBar(Self as PM_Frame ptr)
	dim as GtkWidget ptr menu_bar, menu_item, menu
	
	menu_bar=gtk_menu_bar_new()
	
	menu=gtk_menu_new()

	' Project Menu
	menu_item=gtk_menu_item_new_with_label(t_("Open Project..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Open_Project), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)

	menu_item=gtk_menu_item_new_with_label(t_("Close Project"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Close_Project), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new()
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)

	menu_item=gtk_menu_item_new_with_label(t_("Configure Project..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Configure_Project), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new()
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Refresh Project"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Refresh_Project), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new()
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Exit"))
	g_signal_connect(G_OBJECT(menu_item), "activate", g_CALLBACK(@PMF_Close), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Project"))
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item)
	
	' File Menu
	menu=gtk_menu_new()

	menu_item=gtk_menu_item_new_with_label(t_("New File..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_NewFile), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)

	menu_item=gtk_menu_item_new_with_label(t_("Delete File"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_DeleteFile), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)

	menu_item=gtk_menu_item_new_with_label(t_("Rename File..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_RenameFile), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("File"))
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item)

	' Build Menu
	menu=gtk_menu_new()

	menu_item=gtk_menu_item_new_with_label(t_("Compile"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Build), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Run"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Run), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Debug"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Debug), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)	
	
	menu_item=gtk_menu_item_new_with_label(t_("Building"))
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item)

	' Tools Menu
	menu=gtk_menu_new()
	
	menu_item=gtk_menu_item_new_with_label(t_("TODO Search"))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_Todo), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Search in files..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_FileSearch), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("Tools"))
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item)
	
	' Help Menu
	menu=gtk_menu_new()
	
	menu_item=gtk_menu_item_new_with_label(t_("About this application..."))
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(@PMF_About), Self)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item)
	
	menu_item=gtk_menu_item_new_with_label(t_("About"))
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu)
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item)
	
	' Final steps
	Self->vbox=gtk_vbox_new(0,0)
	gtk_container_add(GTK_CONTAINER(self->frame), self->vbox)
	gtk_box_pack_start(GTK_BOX(self->vbox), menu_bar, 0, 0, 0)
end sub

' ----------------------------------------------------
' Crea un nuovo treeview con le colonne per 
' rappresentare i file
' ----------------------------------------------------
function PMF_Create_Tree_View(self as PM_Frame ptr, byref store as GtkTreeStore ptr, byref tv as GtkWidget ptr) as GtkWidget ptr
	dim as GtkWidget ptr scroll
	dim as GtkCellRenderer ptr renderer
	
	tv=gtk_tree_view_new()
	
	' Colonne
	renderer=gtk_cell_renderer_text_new()
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tv), -1, T_("File Name"), renderer, "text", COL_FILE_NAME, 0)
		
	renderer=gtk_cell_renderer_text_new()
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tv), -1, T_("Type"), renderer, "text", COL_TYPE, 0)
	
	' Store
	store=gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING)
	gtk_tree_view_set_model(GTK_TREE_VIEW(tv), GTK_TREE_MODEL(store))
	
	' Callback
	g_signal_connect_after(G_OBJECT(tv), "row-activated", G_CALLBACK(@PMF_ButtonPress), Self)
	g_object_unref(store)
	
	' Scroll
	scroll=gtk_scrolled_window_new(0,0)
	gtk_container_add(GTK_CONTAINER(scroll), tv)
	
	return scroll
end function

' ----------------------------------------------------
' Crea un nuovo treeview con le colonne per 
' rappresentare i file
' ----------------------------------------------------
function PMF_Create_Tree_View_List(self as PM_Frame ptr, byref store as GtkListStore ptr, byref tv as GtkWidget ptr) as GtkWidget ptr
	dim as GtkWidget ptr scroll
	dim as GtkCellRenderer ptr renderer
	
	tv=gtk_tree_view_new()
	
	' Colonne
	renderer=gtk_cell_renderer_text_new()
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tv), -1, T_("File Name"), renderer, "text", COL_FILE_NAME, 0)
		
	renderer=gtk_cell_renderer_text_new()
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tv), -1, T_("Type"), renderer, "text", COL_TYPE, 0)
	
	' Store
	store=gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING)
	gtk_tree_view_set_model(GTK_TREE_VIEW(tv), GTK_TREE_MODEL(store))
	
	' Callback
	g_signal_connect_after(G_OBJECT(tv), "row-activated", G_CALLBACK(@PMF_ButtonPress), Self)
	g_object_unref(store)
	
	' Scroll
	scroll=gtk_scrolled_window_new(0,0)
	gtk_container_add(GTK_CONTAINER(scroll), tv)
	
	return scroll
end function

' ----------------------------------------------------
' Crea un nuovo treeview con le colonne per 
' rappresentare i log di un processo
' ----------------------------------------------------
function PMF_Create_Tree_View_Log(self as PM_Frame ptr, byref store as GtkListStore ptr, byref tv as GtkWidget ptr) as GtkWidget ptr
	dim as GtkWidget ptr scroll
	dim as GtkCellRenderer ptr renderer
	
	tv=gtk_tree_view_new()
	
	' Colonne
	renderer=gtk_cell_renderer_text_new()
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tv), -1, T_("Log Line"), renderer, "text", COL_FILE_NAME, 0)
		
	' Store
	store=gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING)
	gtk_tree_view_set_model(GTK_TREE_VIEW(tv), GTK_TREE_MODEL(store))
	
	' Callback
	g_signal_connect_after(G_OBJECT(tv), "row-activated", G_CALLBACK(@PMF_ButtonPress_Log), Self)
	g_object_unref(store)
	
	' Scroll
	scroll=gtk_scrolled_window_new(0,0)
	gtk_container_add(GTK_CONTAINER(scroll), tv)
	
	return scroll
end function

' ----------------------------------------------------
' Inizializzazione del modello dell'albero
' ----------------------------------------------------
sub PMF_Tree_Init(self as PM_Frame ptr)
	dim as GtkWidget ptr scrollSources, scrollIncludes, scrollCompilerMessages, scrollDocs, _
		lbSources, lbIncludes, lbCompilerMessages, lbDocs
	
	' Modello e vista sorgenti
	scrollSources=PMF_Create_Tree_View(self, self->storeSources, self->tvSources)
	
	' Modello e vista include
	scrollIncludes=PMF_Create_Tree_View(self, self->storeIncludes, self->tvIncludes)
	
	' Modello e vista docs
	scrollDocs=PMF_Create_Tree_View_List(self, self->storeDocs, self->tvDocs)
	
	' Modello e vista messaggi di compilazione
	scrollCompilerMessages=PMF_Create_Tree_View_Log(self, self->storeCompilerMessages, self->tvCompilerMessages)
		
	' Notebook
	self->notebook=gtk_notebook_new()
	
	lbSources=gtk_label_new(0)
	gtk_label_set_markup(GTK_LABEL(lbSources), "<b>"+T_("Source")+"</b>")
	lbIncludes=gtk_label_new(0)
	gtk_label_set_markup(GTK_LABEL(lbIncludes), "<b>"+T_("Include")+"</b>")
	lbDocs=gtk_label_new(0)
	gtk_label_set_markup(GTK_LABEL(lbDocs), "<b>"+T_("Documents")+"</b>")
	
	lbCompilerMessages=gtk_label_new(T_("Tools Output"))
	
	gtk_notebook_append_page(GTK_NOTEBOOK(self->notebook), scrollSources, lbSources)
	gtk_notebook_append_page(GTK_NOTEBOOK(self->notebook), scrollIncludes, lbIncludes)
	gtk_notebook_append_page(GTK_NOTEBOOK(self->notebook), scrollDocs, lbDocs)
	gtk_notebook_append_page(GTK_NOTEBOOK(self->notebook), scrollCompilerMessages, lbCompilerMessages)
	
	gtk_box_pack_end(GTK_BOX(self->vbox), self->notebook, 1, 1, 2)
end sub

' ----------------------------------------------------
' Inizializzazione della finestra da utilizzare per il
' Project Manager
' ----------------------------------------------------
sub PMF_Init(Self as PM_Frame ptr)
	self->frame=gtk_window_new(GTK_WINDOW_TOPLEVEL)
	self->titolo=t_("Project Manager - v. "+PROJMAN_VER)
	P_Init(@self->progetto)
	
	gtk_window_set_default_size(GTK_WINDOW(self->frame), 550, 400)
	gtk_window_set_title(GTK_WINDOW(self->frame), self->titolo)
	g_signal_connect(G_OBJECT(self->frame), "delete_event", G_CALLBACK(@PMF_Close), @Self)

	PMF_MenuBar(Self)
	PMF_Toolbar_Init(Self)
	PMF_Tree_Init(Self)
	
	gtk_widget_show_all(self->frame)
	gtk_window_move(GTK_WINDOW(self->frame), 0, 0)
end sub

