'    Copyright 2007, Leonardo Cecchi
'
'    This file is part of Project Manager.
'
'    Project Manager is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    Project Manager is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with Project Manager; if not, write to the Free Software
'    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include once "PM_Frame.bi"
#include once "PM_InputDialog.bi"
#include once "PM_SearchDialog.bi"
#include once "BE_Utils.bi"

' ----------------------------------------------------
' Ritorna il tree corrente in base al tab selezionato
' ----------------------------------------------------
function PMF_CurrentTree (self as PM_Frame ptr) as GtkWidget ptr
	dim as Integer t=gtk_notebook_get_current_page(GTK_NOTEBOOK(self->notebook))
	if t=0 then
		return self->tvSources
	elseif t=1 then
		return self->tvIncludes
	elseif t=2 then
		return self->tvDocs
	else
		return 0
	end if
end function

' ----------------------------------------------------
' Rename file
' ----------------------------------------------------
sub PMF_RenameFile cdecl (byref widget as GtkWidget ptr, self as PM_Frame ptr)
	dim as GtkTreeSelection ptr selection
	dim as GtkTreeModel ptr model
	dim as GtkTreeIter iter
	dim as GtkWidget ptr tree
	dim selezionato as String = ""	
	
	tree=PMF_CurrentTree(self)
	if tree=0 then
		return
	end if
	
	selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(tree))
	if gtk_tree_selection_get_selected(selection, @model, @iter) then
		dim p as zstring ptr
		gtk_tree_model_get(model, @iter, COL_FILE_NAME, @p, -1)
		selezionato=selezionato+(*p)
		g_free(p)
	else
		InformationDialog(T_("Please choose a file"))
		return
	end if
	
	if len(selezionato)<>0 then
		dim comando as String
		dim editor as String
		dim d as InputDialog
		ID_Init(@d, self->frame, T_("Rename to:"), T_("Renaming")+" "+selezionato+" ...")
		ID_SetDefault(@d, selezionato)
		ID_Run(@d)
		if len(d.text)=0 then
			ID_Destroy(@d)
			return
		end if	
		
		name selezionato, d.text
		PMF_Refresh_Project(widget, self)
		ID_Destroy(@d)
	end if
end sub

' ----------------------------------------------------
' Delete file
' ----------------------------------------------------
sub PMF_DeleteFile cdecl (byref widget as GtkWidget ptr, self as PM_Frame ptr)
	dim as GtkTreeSelection ptr selection
	dim as GtkTreeModel ptr model
	dim as GtkTreeIter iter
	dim as GtkWidget ptr tree
	dim selezionato as String = ""
	
	
	tree=PMF_CurrentTree(self)
	if tree=0 then
		return
	end if
	
	selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(tree))
	if gtk_tree_selection_get_selected(selection, @model, @iter) then
		dim p as zstring ptr
		gtk_tree_model_get(model, @iter, COL_FILE_NAME, @p, -1)
		selezionato=selezionato+(*p)
		g_free(p)
	else
		InformationDialog(T_("Please choose a file"))
		return
	end if
	
	if len(selezionato)<>0 then
		dim comando as String
		dim editor as String
		if YesNoDialog(T_("Do you really want to delete")+" "+selezionato+"?") then
			kill(selezionato)
			PMF_Refresh_Project(widget, self)
		end if
	end if
end sub

' ----------------------------------------------------
' New file
' ----------------------------------------------------
sub PMF_NewFile cdecl (byref widget as GtkWidget ptr, self as PM_Frame ptr)
	dim as GtkWidget ptr dialog
	
	if P_Empty(@self->progetto) then
		InformationDialog(T_("Please choose a project"))
		return
	end if
	
	chdir(self->progetto.projectDirectory)
	dialog=gtk_file_chooser_dialog_new(t_("New file"), GTK_WINDOW(self->frame), _
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 0)
	if gtk_dialog_run(GTK_DIALOG(dialog))=GTK_RESPONSE_ACCEPT then
		dim as String selected 
		selected=""
		selected+=*gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog))
		if not FileExists(selected) then
			CreateEmptyFile(selected)
			PMF_Refresh_Project(widget, self)
		end if
	end if
	gtk_widget_destroy(dialog)
end sub

' ----------------------------------------------------
' Ricerca la stringa nei file
' ----------------------------------------------------
sub PMF_FileSearch cdecl (byref widget as GtkWidget ptr, self as PM_Frame ptr)
	if not P_Empty(@self->progetto) then
		dim s as SearchDialog
		SD_Init(@s, self->frame)
		if SD_Run(@s)=GTK_RESPONSE_ACCEPT then
			if len(s.text)<>0 then
				P_Grep_All_Files(@self->progetto, s.text)
				PMF_RefreshOutputFromProject(self)
				gtk_notebook_set_current_page(GTK_NOTEBOOK(self->notebook), 3)
			end if
		end if
		SD_Destroy(@s)
	else
		InformationDialog(T_("Please choose a project"))
	end if
end sub

' ----------------------------------------------------
' Riempie la todo list
' ----------------------------------------------------
sub PMF_Todo cdecl (byref widget as GtkWidget ptr, self as PM_Frame ptr)
	if not P_Empty(@self->progetto) then
		P_Todo_Grepper(@self->progetto)
		PMF_RefreshOutputFromProject(self)
		gtk_notebook_set_current_page(GTK_NOTEBOOK(self->notebook), 3)
	else
		InformationDialog(T_("Please choose a project"))
	end if
end sub
